;=======================================;
;Title: DBZ: Assault of the Saiyans 	;
;Huffman-Compatible Text Read		;
;=======================================;
;The original text read routine		;
;rewritten so that it works with the	;
;huffman decoder.			;
;=======================================;

.org $fa66
.mem 8
.index 8

get_current_char:
	jsr decode		;Decode the byte and retrieve the current character.
	beq eos_jump		;If it's an end-of-string control code (#$00 in this game)
				;branch to the end of the routine.

line_break:
	cmp #$02		;Is the current character a line break control code?
	bne spec_space		;If not go test the next control code.

	lda $53			;Load the current least significant byte of the VRAM
	clc			;address to write to, and add $20 to it. Remember $20 = 32
	adc #$20		;the equivalent of 1 complete line on the NES.
	sta $51			;(32 tiles per line)
	sta $53

calc_vram_msb:
	lda $54			;Load the current most significant byte of the VRAM
	adc #$00		;address to write to, and add the carry (if any) from
	sta $54			;the previous line break calculation.
	and #$7f		;If the result isn't positive; force it positive.
	sta $52
	jmp inc_char_count

spec_space:
	cmp #$03		;The "special space" control code, simply increments VRAM
	bne ram_set_up		;address to write to by 1, and doesn't load any tiles to be
	inc $51			;displayed.
	jmp get_current_char

ram_set_up:
	jsr $cac4		;Performs a number of operations to properly set the X
				;register up. Though this routine isn't very important
				;at all upon first inspection, I've later discovered this
				;routine is soley responsible for preparing RAM so that
				;6 characters may be loaded into VRAM at once upon the
				;execution of an A button press.
				;(The "auto-message" feature that increases the rate at
				;which text is displayed when the A button is pressed.

	lda #$02		;Used as a test condition in the nametable load routine.
	sta $0600,x		;This will be used to determine how many writes need to be
				;made to VRAM.

				;Note that this is left unchanged even though
				;we will only be writing ONCE to VRAM when dealing with text,
				;and not twice as this is set up to do. The reason for this
				;is not made fully apparent here, but will be upon examination
				;of the VRAM load routine.

	lda #$04		;Used to set up the PPU later.
	sta $0601,x		;This data will be used to set up the mirroring information
				;during the VRAM load routine.

	lda #$00		;Used to allow us to exit the nametable load routine.
	sta $0606,x		;Specifically, it's used to answer the question, "Are we
				;finished loading data into VRAM?"

	lda $51			;Load/store the VRAM address to write to in the nametable load
	sta $0602,x		;routine.
	lda $52
	sta $0603,x

	lda #$01		;Set an unused byte of RAM to 1 to be used to branch to the new
	sta $06ff		;text display routine in the nametable load routine.

	stx $06fe		;X will be used as a test condition in the VRAM load routine.

variable_check:
	lda $84			;Reload the current character and check
	cmp #$e0		;if its a control code for a variable.
	bcs jump		;We can't do a branch directly to the routine at $d6f6, so
				;we branch to a pseudo-jump table, which isn't really a table
				;but one jmp statement, but you get the idea.

	jsr $d7f0		;At this point we've determined that the current character isn't
				;a line break, a "special space", or a variable. Now, we go to
				;the DTE routine and load the character into RAM so that it maybe
				;loaded into VRAM later. Of course, if it's not a DTE value, it
				;will simply be loaded into RAM and the DTE-specific portion
				;of the code will be skipped entirely.

lsb_inc:
	inc $51			;Increment the LSB of the VRAM addy.

inc_char_count:
	dec $58			;Decrement the number of characters to load into RAM.
				;Note: only one character is loaded at a time UNLESS
				;the A button is pressed, in which case a maximum of
				;6 characters are loaded into RAM, and $58 will be set
				;accordingly to allow for this.

	beq bank_switch		;If $58 == 0, we're finished reading in characters for the
				;time being.

	jmp get_current_char	;If $58 != 0, we've got more characters to load into RAM.

bank_switch:
	pla			;Swap the 16kb bank at $8000 out for what was loaded prior
	sta $8008		;to the bank containing the game dialogue.
	jmp $d6a3		;Jump to the RTS statement at the end of the original
				;text read routine.

jump:
	jmp $d6a4		;Jump to the Variable control code routine.

eos_jump:
	jmp $fd67

;=======================================;
;Title: DBZ: Assault of the Saiyans 	;
;Huffman Text Decoder			;
;=======================================;
;Decodes huffman encoded text that's	;
;been implemented in both scripts.	;
;=======================================;

decode:
	lda $82			;In order for the variable control codes to work properly
	sta $06fc		;we need to preserve the bit counter...

	lda $83			;The WIP character being shifted...
	sta $06fd

	lda $50			;And the load index.
	sta $06fb

table_set:
	lda #$19		;Look-up table starts at $3fb29 in .NES file.
	sta $80

	lda #$fb
	sta $81
	ldx $82

check:
	ldy #$00		;Start at the top of the table.
	lda ($80),y
	beq leaf_found		;If the first byte is #$00, then the next
				;byte is the decoded value.

	dex			;Decrement X (count) to determine how
	bpl bit_shift		;how many bits left in current byte.
				;Sign bit will always be cleared (plus) unless count was #$00 before
				;the preceding dex.

get_byte:
	ldx #$7			;7 bits per byte (starting at 0).

	ldy $50
	lda ($55),y
	sta $83

	inc $50			;Increment index so the right byte is read next time.
	ldy #$00		;Load Y with #$00 so we don't use $50 in the node pointer arthimetic ahead.

bit_shift:
	asl $83			;Shift the next bit to be tested into the Carry flag.
	bcc next_level		;If C equals 0, follow the left branch; if C equals 1, follow the right.

	iny			;Access the right branch.

next_level:
	lda ($80),y		;The relative address to the next node.

	clc			;Add the relative address of the next node
	adc $80			;to the current node pointer.
	sta $80
	bcc check

	inc $81
	bcs check

leaf_found:
	iny
	lda ($80),y		;The decoded value
	stx $82
	sta $84			;Store the decoded value in RAM so we don't have to disturb the "stock" operations
				;too much.
	rts			;Run the decoded value through the normal
				;text routine.

;=======================================;
;Title: DBZ: Assault of the Saiyans 	;
;Huffman End of String Set Up		;
;=======================================;
;Fixes the End of String bug.		;
;=======================================;

.org $fd67

eos:
	lda $06fc
	sta $82

	lda $06fd
	sta $83

	lda $06fb
	sta $50

	pla			;Swap the 16kb bank at $8000 out for what was loaded prior
	sta $8008		;to the bank containing the game dialogue.
	jmp $d6a3		;Jump to the RTS statement at the end of the original
				;text read routine.
